#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <sys/socket.h>
#include <sys/ioctl.h>

#include <net/if.h>
#include <linux/if_lac.h>//ADI
#include <linux/net_switch.h>
#include <linux/sockios.h>

#include "lac_user.h"


void print_header()
{
}

void lac_usage()
{
	print_header();
	
	printf("swctl agg name add/rem               Add/Remove aggregator\n");
	printf("swctl port name add/rem              Add/Remove port to/from system\n\n");

	printf("swctl aggregation port_no ena/dis    Set port aggregation (A/I)\n");
	printf("swctl mode port_no active/passive    Set port lacp mode\n");
	printf("swctl key port_no value              Set port admin key <1 - 65535>\n");
	printf("swctl port_prio port_no value        Set port priority <1 - 65535>\n");
	printf("swctl timeout port_no value          Set port timeout <long/short>\n\n");

	printf("swctl sys_prio value                 Set system priority <1 - 65535>\n");
	printf("swctl sys_max_delay value            Set system collector max delay <1 - 65536>\n\n");
	
	printf("swctl show port                      Show ports' parameters\n");
	printf("swctl show agg                       Show aggregators\n");
	printf("swctl show system                    Show system parameters\n\n");
	
	printf("swctl debug 1/2/3/4                  Toggle debug (rx/tx/state/warnings)\n");
	printf("swctl debug port port_no             Toggle debug for port\n\n");
}

int check_port(int port_no)
{
	/* if port_no < min_port && port_no > max_port)
	   {
			printf("Wrong port selected\n");
			return -1;
	   }
	*/ 

	return 0;
}

int send_agg_command(int sock, struct net_switch_ioctl_arg user_arg)
{
	int status;
	if(user_arg.cmd==0)
	{
		printf("ERR lac_user.c send_agg_command; nu e setata user_arg.cmd, ci e zero; NAT\n ");
		exit(1);
	}

	status = ioctl(sock, SIOCSWCFG, &user_arg);
	if (status)
	{
		perror("command failed \n");
		return -1;
	}
	
	printf("command succeded\n");

	return 0;
}


/**
 *	Add/Remove aggregator command
 */
int addrem_agg(int sock, int argc, char **argv)
{
	int status;
	int  size;
	struct net_switch_ioctl_arg user_arg;
	
	if (argc < 4)
		goto wrong_command;


	user_arg.if_name=(char*)malloc(IFNAMSIZ*sizeof(char));
	memset(user_arg.if_name,0,IFNAMSIZ*sizeof(char));
	strncpy(user_arg.if_name,argv[2],(IFNAMSIZ-1)*sizeof(char));
	
	if (!strcmp(argv[3], "add"))
	{
		/* add aggregator command */
		user_arg.cmd=LACTL_ADD_AGG;
		//status = ioctl(sock, SIOCLACADDAGG, argv[2]);
		status = ioctl(sock, SIOCSWCFG, &user_arg);
	}
	else if (!strcmp(argv[3], "rem"))
	{
		user_arg.cmd=LACTL_REM_AGG;
		//status = ioctl(sock, SIOCLACDELAGG, argv[2]);  
		status = ioctl(sock, SIOCSWCFG, &user_arg);  
	}

	if (status)
	{
		perror("add/rem aggregator failed\n");
		free(user_arg.lac_user_cmd.param);
		return -1;
	}

	printf("add/rem aggregator command succeded\n");

	return 0;

wrong_command:
	return -1;

}

/**
 *	Port configuration commands
 */
int cfg_port_cmd(int sock, int command, int argc, char **argv)
{
	unsigned int port_no = 0, value = 0;
	lac_cfg_cmd_t cfg_cmd;
	struct net_switch_ioctl_arg user_arg;

	if (argc < 4)
		goto wrong_command;

	if (command != LACTL_IF)
	{
		/* get port number */
		port_no = atoi(argv[2]);
		if (check_port(port_no) < 0)
		{
			printf("wrong port\n");	
			return -1;
		}
	}

	switch (command)
	{
	case LACTL_IF:
		/* add/remove port to/from lacp */
		if (!strcmp(argv[3], "add"))
			user_arg.cmd= LACTL_ADD_IF;
		else if (!strcmp(argv[3], "rem"))
			user_arg.cmd= LACTL_DEL_IF;
		else
			goto wrong_command;
		break;
	case LACTL_SET_AGG_STATE:
		/* set aggregation state - enabled/disabled */
		if (!strcmp(argv[3], "ena"))
			user_arg.cmd= LACTL_SET_AGG_STATE_ENABLED;
		else if (!strcmp(argv[3], "dis"))
			user_arg.cmd= LACTL_SET_AGG_STATE_DISABLED;
		else
			goto wrong_command;
		break;
	case LACTL_SET_MODE:
		/* set port mode - active/passive */
		if (!strcmp(argv[3], "active"))
			user_arg.cmd= LACTL_SET_MODE_ACTIVE;
		else if (!strcmp(argv[3], "passive"))
			user_arg.cmd= LACTL_SET_MODE_PASSIVE;
		else
			goto wrong_command;
		break;
	case LACTL_SET_ADMIN_KEY:
	case LACTL_SET_PORT_PRIO:
		/* set port admin key */
		value = atoi(argv[3]);
		/* check range */
		if (value < 0 || value > LAC_MAX_KEY_PRIO)
		{
			printf("Value out of range\n");
			return -1;
		}

		cfg_cmd.value = value;
		user_arg.cmd= command;
		break;
	case LACTL_SET_TIMEOUT:
		if (!strcmp(argv[3], "short"))
			user_arg.cmd= LACTL_SET_SHORT_TIMEOUT;
		else if (!strcmp(argv[3], "long"))
			user_arg.cmd= LACTL_SET_LONG_TIMEOUT;
		else
			goto wrong_command;
		break;
	default:
		goto wrong_command;
	}

	/* command */
	if (command != LACTL_IF)
		cfg_cmd.port.port_no = port_no;
	else
		cfg_cmd.port.port_name = argv[2];
	user_arg.lac_user_cmd.param = (void *)&cfg_cmd;

	user_arg.if_name=(char*)malloc(IFNAMSIZ*sizeof(char));
	memset(user_arg.if_name,0,IFNAMSIZ*sizeof(char));
	strncpy(user_arg.if_name,DEFAULT_AGG_NAME,(IFNAMSIZ-1)*sizeof(char));

	/* send command */
	return send_agg_command(sock, user_arg);

wrong_command:
	/* wrong user command */
	return -1;
}

/**
 *	System configuration commands
 */
int cfg_sys_cmd(int sock, int command, int argc, char **argv)
{
	unsigned int value;
	lac_cfg_cmd_t cfg_cmd;
	struct net_switch_ioctl_arg user_arg;
	
	if (argc < 3)
		goto wrong_command;

	switch (command)
	{
	case LACTL_SET_SYS_PRIO:
	case LACTL_SET_SYS_MAX_DELAY:
		value = atoi(argv[2]);
		user_arg.cmd= command;
		cfg_cmd.value = value;
		break;
	default:
		goto wrong_command;
	}

	/* command */
	user_arg.lac_user_cmd.param = (void *)&cfg_cmd;

	user_arg.if_name=(char*)malloc(IFNAMSIZ*sizeof(char));
	memset(user_arg.if_name,0,IFNAMSIZ*sizeof(char));
	strncpy(user_arg.if_name,DEFAULT_AGG_NAME,(IFNAMSIZ-1)*sizeof(char));

	/* send command */
	return send_agg_command(sock, user_arg);

wrong_command:
	return -1;
}

/**
 *	Show system params
 */
void print_system(sys_params_t sp)
{
	printf("-------------------------------------------------------------\n");
	printf("|    System  MAC    | System Priority | Collector Max Delay |\n");
	printf("-------------------------------------------------------------\n");
	printf("| %02x.%02x.%02x.%02x.%02x.%02x | %15d | %19d |\n", 
		sp.sys_id[0], sp.sys_id[1], sp.sys_id[2], sp.sys_id[3],
		sp.sys_id[4], sp.sys_id[5],
		sp.sys_prio,
		sp.collector_max_delay);
	printf("-------------------------------------------------------------\n\n");
}

void show_system(int sock)
{
	lac_cmd_t cmd;
	sys_params_t sys_params;
	struct net_switch_ioctl_arg user_arg;

	/* command */
	cmd.param = (void *)&sys_params;

	user_arg.if_name=(char*)malloc(IFNAMSIZ*sizeof(char));
	memset(user_arg.if_name,0,IFNAMSIZ*sizeof(char));
	strncpy(user_arg.if_name,DEFAULT_AGG_NAME,(IFNAMSIZ-1)*sizeof(char));

	user_arg.cmd=LACTL_GET_SYSTEM;
	user_arg.lac_user_cmd=cmd;

	/* send command */
	if (send_agg_command(sock, user_arg) == 0)
	{
		/* print system params */
		print_system(sys_params);
	}
}

/**
 *	Print port params
 */
void print_port_table_header()
{
	 printf("--------------------------------------------------------------------------------\n");
	 printf("|     |      |            |       |     |       | Admin |  Oper | Part |       |\n");
	 printf("| No. | Name | Agg / Name |  Prio | A/I | T-out |  Key  |   Key |  No  | State |\n");  
	 printf("--------------------------------------------------------------------------------\n");
}

void print_port_table_footer()
{
	 printf("--------------------------------------------------------------------------------\n\n");
}

void print_port_params(port_params_t pp)
{
	printf("| %3d | %4s | %2d (%5s) | %5d | %3c | %5s | %5d | %5d | %4d |%6s |\n",
		pp.port_no,
		pp.port_name,
		pp.agg_no,
		pp.agg_name,
		pp.actor_port_prio,
		(pp.ai ? 'A' : 'I'),
		(pp.timeout ? "Short" : "Long"),
		pp.actor_admin_key,
		pp.actor_oper_key,
		pp.part_port_no,
		(pp.port_status ? "up" : "down"));

}

#if 0
void print_port_params(port_params_t pp)
{
	printf("========================================================\n");
	printf("port number: %d\n", pp.port_no);
	printf("port name: %s\n", pp.port_name);
	
	printf("aggregator: %d\n", pp.agg_no);
	printf("agg name: %s\n", pp.agg_name);

	printf("port status: %d\n", pp.port_status);
	printf("ai: %d\n", pp.ai);
	printf("timeout: %d\n", pp.timeout);

	printf("actor id: %02x.%02x.%02x.%02x.%02x.%02x\n", pp.actor_id[0], pp.actor_id[1],
		pp.actor_id[2], pp.actor_id[3], pp.actor_id[4], pp.actor_id[5]);
	printf("actor admin key: %d\n", pp.actor_admin_key);
	printf("actor oper key: %d\n", pp.actor_oper_key);
	printf("actor port prio: %d\n", pp.actor_port_prio);
	printf("actor state: %d\n", pp.actor_state);
	
	printf("part sys id: %02x.%02x.%02x.%02x.%02x.%02x\n", pp.part_sys_id[0], pp.part_sys_id[1],
		pp.part_sys_id[2], pp.part_sys_id[3], pp.part_sys_id[4], pp.part_sys_id[5]);
	printf("part admin key: %d\n", pp.part_admin_key);
	printf("part oper key: %d\n", pp.part_oper_key);
	printf("part port prio: %d\n", pp.part_port_prio);
	printf("part state: %d\n", pp.part_state);
	printf("========================================================\n");
}
#endif

void show_ports(int sock)
{
	lac_cmd_t cmd;
	port_params_t ports_params[LAC_MAX_PORTS];
	//struct ifreq req;
	struct net_switch_ioctl_arg user_arg;

	memset(ports_params, 0, LAC_MAX_PORTS*sizeof(port_params_t));

	/* command */
	cmd.param = (void *)&ports_params;

	user_arg.if_name=(char*)malloc(IFNAMSIZ*sizeof(char));
	memset(user_arg.if_name,0,IFNAMSIZ*sizeof(char));
	strncpy(user_arg.if_name,DEFAULT_AGG_NAME,(IFNAMSIZ-1)*sizeof(char));

	user_arg.cmd=LACTL_GET_PORTS;
	user_arg.lac_user_cmd=cmd;

	/* send command */
	if (send_agg_command(sock, user_arg) == 0)
	{
		int i;

		/* header */
		print_port_table_header();

		/* print port params */
		for (i = 0; i < LAC_MAX_PORTS; i++)
		{
			if (ports_params[i].port_no < 1)
				break;

			print_port_params(ports_params[i]);
		}

		/* footer */
		print_port_table_footer();
	}
}

port_params_t get_port_by_name(int sock,char *name)
{
	lac_cmd_t cmd;
	port_params_t ports_params[1];
	struct net_switch_ioctl_arg user_arg;
	memset(ports_params, 0,sizeof(port_params_t));

	strncpy(ports_params[0].port_name,name,MAX_PORT_NAME);

	/* command */
	cmd.param = (void *)&ports_params;

	user_arg.if_name=(char*)malloc(IFNAMSIZ*sizeof(char));
	memset(user_arg.if_name,0,IFNAMSIZ*sizeof(char));
	strncpy(user_arg.if_name,DEFAULT_AGG_NAME,(IFNAMSIZ-1)*sizeof(char));

	user_arg.cmd=LACTL_GET_PORT_BY_NAME;
	user_arg.lac_user_cmd=cmd;

	/* send command */
	if (send_agg_command(sock, user_arg) == 0)
	{
		return  ports_params[0];
	}
	ports_params[0].port_no=-1;//adica oricum a esuat
	return ports_params[0];
}

/**
 *	Print aggregator params
 */
void print_agg_table_header()
{
	printf("---------------------------------------------------\n");
	printf("|     |        |       |       | Distrib. |       |\n");
	printf("| No. |  Name  |  Key  | Ports | Ports    | Ready |\n");  
	printf("---------------------------------------------------\n");
}

void print_agg_table_footer()
{
	printf("---------------------------------------------------\n\n");
}

void print_agg_params(agg_params_t ap)
{
	printf("| %3d | %6s | %5d | %5d | %8d | %5s |\n",
		ap.agg_no,
		ap.agg_name,
		ap.agg_key,
		ap.nports,
		ap.agg_distrib_ports,
		(ap.ready ? "true" : "false"));
}

void show_aggregators(int sock)
{
	lac_cmd_t cmd;
	agg_params_t agg_params[LAC_MAX_AGGS];
	struct net_switch_ioctl_arg user_arg;

	memset(agg_params, 0, LAC_MAX_AGGS*sizeof(agg_params_t));

	/* command */ 
	cmd.param = (void *)&agg_params;

	user_arg.if_name=(char*)malloc(IFNAMSIZ*sizeof(char));
	memset(user_arg.if_name,0,IFNAMSIZ*sizeof(char));
	strncpy(user_arg.if_name,DEFAULT_AGG_NAME,(IFNAMSIZ-1)*sizeof(char));

	user_arg.cmd=LACTL_GET_AGG;
	user_arg.lac_user_cmd=cmd;

	/* send command */
	if (send_agg_command(sock, user_arg) == 0)
	{
		int i;

		/* header */
		print_agg_table_header();

		for (i = 0; i < LAC_MAX_AGGS; i++)
		{
			if (agg_params[i].agg_no < 1)
				break;

			print_agg_params(agg_params[i]);
		}

		/* footer */
		print_agg_table_footer();
	}
}

/**
 *	Show commands
 */
int show_cmd(int sock, int argc, char **argv)
{
	if (argc < 3)
		goto wrong_command;

	if (!strcmp(argv[2], "port"))
	{
		/* show ports info */
		show_ports(sock);
	}
	else if (!strcmp(argv[2], "agg"))
	{
		/* show aggs info */
		show_aggregators(sock);
	}
	else if (!strcmp(argv[2], "system"))
	{
		/* show system params */
		show_system(sock);

	}
	else if (!strcmp(argv[2], "stats"))
	{
		/* show lacp stats */
	
	}
	else
		goto wrong_command;

	return 0;

wrong_command:
	return -1;
}

/**
 *	Debug commands
 */
int debug_cmd(int sock, int argc, char **argv)
{
	unsigned int port_no, machine;
	lac_cfg_cmd_t cfg_cmd;
	lac_cmd_t cmd;  
	struct net_switch_ioctl_arg user_arg;

	if (argc < 3)
		goto wrong_command;

	if (!strcmp(argv[2], "port"))
	{
		/* port debug command */
		printf("port debug command\n");

		if (argc < 4)
			goto wrong_command;
		
		/* get port number */
		port_no = atoi(argv[3]);
		if (check_port(port_no) < 0)
			goto wrong_command;

		/* cfg_cmd */
		cfg_cmd.port.port_no = port_no;

		/* command */
		user_arg.cmd=LACTL_DEBUG_PORT;
		cmd.param = (void *)&cfg_cmd;
	}
	else 
	{	
		machine = atoi(argv[2]);
		/* machine debug command */
		switch (machine)
		{
		case RX_MACHINE:
			/* rx machine */
			user_arg.cmd= LACTL_DEBUG_RX;
			break;
		case TX_MACHINE:
			/* tx machine */
			user_arg.cmd= LACTL_DEBUG_TX;
			break;
		case STATE:
			/* state */
			user_arg.cmd= LACTL_DEBUG_STATE;
			break;
		case WARN:
			/* warnings */
			user_arg.cmd= LACTL_DEBUG_WARN;
			break;
		default:
			goto wrong_command;
		}
	}

	user_arg.if_name=(char*)malloc(IFNAMSIZ*sizeof(char));
	memset(user_arg.if_name,0,IFNAMSIZ*sizeof(char));
	strncpy(user_arg.if_name,DEFAULT_AGG_NAME,(IFNAMSIZ-1)*sizeof(char));

	user_arg.lac_user_cmd=cmd;

	/* send command */
	return send_agg_command(sock, user_arg);

wrong_command:
	return -1;
}

int lac_cmd(int argc, char *argv[],int sock)
{

	/* command */
	if (!strcmp(argv[1], "agg"))
	{
		/* add/remove aggregator */
		addrem_agg(sock, argc, argv);
	}
	else if (!strcmp(argv[1], "port"))
	{
		/* add/remove port */
		cfg_port_cmd(sock, LACTL_IF, argc, argv);
	}
	else if (!strcmp(argv[1], "aggregation"))
	{
		/* port aggregation - aggregatable/individual */
		cfg_port_cmd(sock, LACTL_SET_AGG_STATE, argc, argv);
	}
	else if (!strcmp(argv[1], "mode"))
	{
		/* port mode - active/passive */
		cfg_port_cmd(sock, LACTL_SET_MODE, argc, argv);
	}
	else if (!strcmp(argv[1], "key"))
	{
		/* port admin key */
		cfg_port_cmd(sock, LACTL_SET_ADMIN_KEY, argc, argv);
	}
	else if (!strcmp(argv[1], "port_prio"))
	{
		/* port priority */
		cfg_port_cmd(sock, LACTL_SET_PORT_PRIO, argc, argv);
	}
	else if (!strcmp(argv[1], "timeout"))
	{
		/* port timeout */
		cfg_port_cmd(sock, LACTL_SET_TIMEOUT, argc, argv);
	}
	else if (!strcmp(argv[1], "sys_prio"))
	{
		/* system priority */
		cfg_sys_cmd(sock, LACTL_SET_SYS_PRIO, argc, argv);
	}
	else if (!strcmp(argv[1], "sys_max_delay"))
	{
		/* system collector max delay */
		cfg_sys_cmd(sock, LACTL_SET_SYS_MAX_DELAY, argc, argv);
	}
	else if (!strcmp(argv[1], "show"))
	{
		/* show command */
		show_cmd(sock, argc, argv);
	}
	else if (!strcmp(argv[1], "debug"))
	{
		/* debug command */
		debug_cmd(sock, argc, argv);
	}
	else
	{
		goto wrong_command;
	}

	return 0;

	wrong_command:
	return -1;

}











